#ifndef PROXY_H
#define PROXY_H

#include "api-driver/stricts-enable.h"
#include <boost/log/trivial.hpp>
#include <terminal_api/ControlProtoCInterface.h>


std::ostream& operator<<(std::ostream& out, CP_Result result);

namespace api_driver::proxy {
    class CpProxy {
    public:
        TSID sid;
        CP_Result lastCpError = OK;
        CpProxy();
        CpProxy(TSID s);

        /**
         * Вызывает процедуру подключения к CP_API. Бросает исключение, если не удалось подключиться.
         */
        void connect();

        /**
         * Процедура отключения от API, после ее вызова запрещается использовать CP_API
         */
        void disconnect();

        std::string getDmaDebug(const std::string& arg);
        void setDmaDebug(const std::string& arg, const std::string& value);

        std::string getNetwork(const std::string& param);
        void setNetwork(const std::string& param, const std::string& value);

        void getModState(modulator_state& dest);
        void getModSettings(modulator_settings& dest);
        void setModSettings(modulator_settings& dest);

        void getDemodState(demodulator_state& dest);
        void getDemodSettings(demodulator_settings& dest);
        void setDemodSettings(demodulator_settings& dest);

        #ifdef API_STRUCT_ACM_ENABLE
        void getAcmSettings(ACM_parameters_serv_& dest);
        void setAcmSettings(ACM_parameters_serv_& dest);
        #endif

        void getDeviceState(device_state& dest);

        #ifdef API_OBJECT_QOS_SETTINGS_ENABLE
        std::tuple<std::string, bool> getQosSettings();
        void setQosSettings(const std::string& rules, bool enable);
        #endif

        #ifdef API_OBJECT_DPDI_SETTINGS_ENABLE
        void getDpdiSettings(DPDI_parmeters& dest);
        void setDpdiSettings(DPDI_parmeters& dest);
        #endif

        #ifdef API_OBJECT_BUCLNB_SETTINGS_ENABLE
        void getBuclnbSettings(buc_lnb_settings& dest);
        void setBuclnbSettings(buc_lnb_settings& dest);
        #endif

        #ifdef MODEM_IS_SCPC
        void getCincState(CinC_state& dest);
        #endif

        #ifdef API_OBJECT_DEBUG_METRICS_ENABLE
        void getDebugMetrics(debug_metrics& dest);
        #endif

        #ifdef MODEM_IS_TDMA
        void getUpdateStatus(progress_msg& dest);
        #endif

        ~CpProxy();
    };
}


#endif //PROXY_H
